/*
OpenFire, a electrical Firework,
easy to build, impressive and absolutely free :-)

License:
"OpenFire" by Jennifer Sirtl (jennifer.sirtl@mostfun.de) is licensed under CC BY 4.0

<a rel="license" href="http://creativecommons.org/licenses/by/4.0/"><img alt="Creative Commons Lizenzvertrag" style="border-width:0" src="https://i.creativecommons.org/l/by/4.0/88x31.png" /></a>
<br />Dieses Werk ist lizenziert unter einer <a rel="license" href="http://creativecommons.org/licenses/by/4.0/">Creative Commons Namensnennung 4.0 International Lizenz</a>.

Explaination:
OpenFire is structured a little like a coconut tree, so you have a tree trunk (ascending/shoot effect) and you have a certain number of tree branches. Most settings are made in config.h, only pin-settings of OpenFire3 must be made in main/OpenFire-file.

NUM_LED_Strip  means, how many LEDs can you count on one tree branch. I choosed 15 because I decide to have 20 tree branches so I needed 300 LEDs.
The density of LED per meter only scale the size of OpenFire. If you choose a 144 LED/m stripe, you get a very small OpenFire (doesn´t look so amazing). On the other hand, if you use a 30LED/meter stripe, you will get a about 1m diameter Effect. 
For bigger size you can use WS2811 chains or you increase the count of LEDs on one tree (but you will need more CPU-Power to compute the increase of LEDs).

NUM_LEDS_Strip_Shoot defines the number of LEDs on the tree trunk (ascending effect). Its independent of the tree branches, but the meaning is like explained. You can choose it depending on the size of the ascending effect. 

>NUM_Strips 20 This number defines the count of the tree trunk. With this Number, you define how many trunks you have, how many wires/ports you will need to contact.

You can also turn OpenFire (top/down) to create Z-Shapes or something else.



REV: 210710 
Ascend Effect reversed, Debug, Test with Teensy LC

*/


#include <Arduino.h>
#include <FastLED.h>       // Fastled library to control the LEDs
#include "config.h"
#include "raise.h"
#include "spheres.h"
#include "crackling.h"
#include "comets.h"

CRGB leds[NUM_LEDS];
CRGB leds_Shoot[NUM_LEDS_Strip_Shoot];

uint32_t Shape[][5]={
	{0xF0F0F0,0x404040,0x202020,0x101010,0x080808},  //00
  {0xF0F0F0,0x501050,0x301030,0x100E10,0x0A0810},  //01
	{0xFF0000,0x808000,0x008000,0x004080,0x0000FF},  //02
	{0xF0F0F0,0x404040,0x0201020,0x00810,0x080408},  //03
	{0xFFD700,0x403500,0x181200,0x030300,0x010100},  //04 Greengold
	{0x200C00,0x140900,0x0A0500,0x050200,0x030100},  //05 Gold
	{0xFFFFFF,0x404040,0x080808,0x040404,0x010101},  //06 Silver
	{0xFF0000,0x400000,0x080000,0x040000,0x010000}   //07 Red	
	};
  #ifdef SOUND_ON
  DFPlayerMini_Fast myDFPlayer;
  #endif
  
void setup() {
  #ifdef DEBUG
    Serial.begin (115200); //Debug option
  #endif
  #ifdef SOUND_ON
    //Initial Player
    Serial1.begin(9600); //Communication to MP3-Player

    #ifdef DEBUG
    Serial.println();
    Serial.println(F("DFRobot DFPlayer Mini Demo"));
    Serial.println(F("Initializing DFPlayer ... (May take 3~5 seconds)"));
    #endif

    //Setup Connection to MP3-Player
    if (!myDFPlayer.begin(Serial1)) {  //Use softwareSerial to communicate with mp3.
      #ifdef DEBUG
      Serial.println(F("Unable to begin:"));
      Serial.println(F("1.Please recheck the connection!"));
      Serial.println(F("2.Please insert the SD card!"));
      #endif
      while(true){
        delay(0); // Code to compatible with ESP8266 watch dog.
      }
    }
    #ifdef DEBUG
    Serial.println(F("DFPlayer Mini online."));
    #endif

    myDFPlayer.volume(0xC0);  //Set volume value. From 0 to 30
   /*     
    myDFPlayer.play(1);  //pfeifen hoch
    myDFPlayer.play(2);  //knall
    myDFPlayer.play(3);  //knall gut
    myDFPlayer.play(4);  //knall
    myDFPlayer.play(5);  //seq gut
    myDFPlayer.play(6);  //pfeifen hoch gut
    myDFPlayer.play(7);  // pfeifen tief gut
    myDFPlayer.play(8);  //pfeifen 
    myDFPlayer.play(9);  //knall crackling gut
    myDFPlayer.play(10);  //sequenz    
    myDFPlayer.play(11);  //seq gut
    myDFPlayer.play(12);  //seq
    myDFPlayer.play(13);  //seq
    myDFPlayer.play(14);  //seq
    myDFPlayer.play(15);  //applaus
    myDFPlayer.play(16);  //crackling
    myDFPlayer.play(17);  //knall crackling 
    myDFPlayer.play(18);  //knall 
    myDFPlayer.play(19);  //knall 
    myDFPlayer.play(20);  //plopp
    myDFPlayer.play(21);  //doppelschuss
    myDFPlayer.play(22);  //Abschuss und plopp
    myDFPlayer.play(23);  //einfachplopp
    myDFPlayer.play(24);  //einfachplopp*/
    #endif

  #ifdef OpenFire3
    //OpenFire3
    //Setup LEDs
    #if LEDTyp == 2812
      FastLED.addLeds<WS2812,34,GRB>(leds, 0*NUM_LEDS_Strip,NUM_LEDS_Strip);  // Init of the Fastled library
      FastLED.addLeds<WS2812,35,GRB>(leds, 1*NUM_LEDS_Strip,NUM_LEDS_Strip);  // Init of the Fastled library
      FastLED.addLeds<WS2812,36,GRB>(leds, 2*NUM_LEDS_Strip,NUM_LEDS_Strip);  // Init of the Fastled library
      FastLED.addLeds<WS2812,37,GRB>(leds, 3*NUM_LEDS_Strip,NUM_LEDS_Strip);  // Init of the Fastled library
      FastLED.addLeds<WS2812,38,GRB>(leds, 4*NUM_LEDS_Strip,NUM_LEDS_Strip);  // Init of the Fastled library
      FastLED.addLeds<WS2812,39,GRB>(leds, 5*NUM_LEDS_Strip,NUM_LEDS_Strip);  // Init of the Fastled library
      FastLED.addLeds<WS2812,40,GRB>(leds, 6*NUM_LEDS_Strip,NUM_LEDS_Strip);  // Init of the Fastled library
      FastLED.addLeds<WS2812,41,GRB>(leds, 7*NUM_LEDS_Strip,NUM_LEDS_Strip);  // Init of the Fastled library
      FastLED.addLeds<WS2812,42,GRB>(leds, 8*NUM_LEDS_Strip,NUM_LEDS_Strip);  // Init of the Fastled library
      FastLED.addLeds<WS2812,43,GRB>(leds, 9*NUM_LEDS_Strip,NUM_LEDS_Strip);  // Init of the Fastled library
      FastLED.addLeds<WS2812,44,GRB>(leds, 10*NUM_LEDS_Strip,NUM_LEDS_Strip);  // Init of the Fastled library
      FastLED.addLeds<WS2812,45,GRB>(leds, 11*NUM_LEDS_Strip,NUM_LEDS_Strip);  // Init of the Fastled library
      FastLED.addLeds<WS2812,46,GRB>(leds, 12*NUM_LEDS_Strip,NUM_LEDS_Strip);  // Init of the Fastled library
      FastLED.addLeds<WS2812,47,GRB>(leds, 13*NUM_LEDS_Strip,NUM_LEDS_Strip);  // Init of the Fastled library
      FastLED.addLeds<WS2812,48,GRB>(leds, 14*NUM_LEDS_Strip,NUM_LEDS_Strip);  // Init of the Fastled library
      FastLED.addLeds<WS2812,49,GRB>(leds, 15*NUM_LEDS_Strip,NUM_LEDS_Strip);  // Init of the Fastled library
      FastLED.addLeds<WS2812,50,GRB>(leds, 16*NUM_LEDS_Strip,NUM_LEDS_Strip);  // Init of the Fastled library
      FastLED.addLeds<WS2812,51,GRB>(leds, 17*NUM_LEDS_Strip,NUM_LEDS_Strip);  // Init of the Fastled library
      FastLED.addLeds<WS2812,52,GRB>(leds, 18*NUM_LEDS_Strip,NUM_LEDS_Strip);  // Init of the Fastled library
      FastLED.addLeds<WS2812,53,GRB>(leds, 19*NUM_LEDS_Strip,NUM_LEDS_Strip);  // Init of the Fastled library
      FastLED.addLeds<WS2812,LED_PIN_Shoot,GRB>(leds_Shoot, NUM_LEDS_Strip_Shoot);  // Init of the Fastled library
    #elif LEDTyp == 2811
      FastLED.addLeds<WS2812,24,RGB>(leds, 0*NUM_LEDS_Strip,NUM_LEDS_Strip);  // Init of the Fastled library
      FastLED.addLeds<WS2812,25,RGB>(leds, 1*NUM_LEDS_Strip,NUM_LEDS_Strip);  // Init of the Fastled library
      FastLED.addLeds<WS2812,26,RGB>(leds, 2*NUM_LEDS_Strip,NUM_LEDS_Strip);  // Init of the Fastled library
      FastLED.addLeds<WS2812,27,RGB>(leds, 3*NUM_LEDS_Strip,NUM_LEDS_Strip);  // Init of the Fastled library
      FastLED.addLeds<WS2812,28,RGB>(leds, 4*NUM_LEDS_Strip,NUM_LEDS_Strip);  // Init of the Fastled library
      FastLED.addLeds<WS2812,29,RGB>(leds, 5*NUM_LEDS_Strip,NUM_LEDS_Strip);  // Init of the Fastled library
      FastLED.addLeds<WS2812,30,RGB>(leds, 6*NUM_LEDS_Strip,NUM_LEDS_Strip);  // Init of the Fastled library
      FastLED.addLeds<WS2812,31,RGB>(leds, 7*NUM_LEDS_Strip,NUM_LEDS_Strip);  // Init of the Fastled library
      FastLED.addLeds<WS2812,32,RGB>(leds, 8*NUM_LEDS_Strip,NUM_LEDS_Strip);  // Init of the Fastled library
      FastLED.addLeds<WS2812,33,RGB>(leds, 9*NUM_LEDS_Strip,NUM_LEDS_Strip);  // Init of the Fastled library
      FastLED.addLeds<WS2812,34,RGB>(leds, 10*NUM_LEDS_Strip,NUM_LEDS_Strip);  // Init of the Fastled library
      FastLED.addLeds<WS2812,35,RGB>(leds, 11*NUM_LEDS_Strip,NUM_LEDS_Strip);  // Init of the Fastled library
      FastLED.addLeds<WS2812,36,RGB>(leds, 12*NUM_LEDS_Strip,NUM_LEDS_Strip);  // Init of the Fastled library
      FastLED.addLeds<WS2812,37,RGB>(leds, 13*NUM_LEDS_Strip,NUM_LEDS_Strip);  // Init of the Fastled library
      FastLED.addLeds<WS2812,38,RGB>(leds, 14*NUM_LEDS_Strip,NUM_LEDS_Strip);  // Init of the Fastled library
      FastLED.addLeds<WS2812,39,RGB>(leds, 15*NUM_LEDS_Strip,NUM_LEDS_Strip);  // Init of the Fastled library
      FastLED.addLeds<WS2812,40,RGB>(leds, 16*NUM_LEDS_Strip,NUM_LEDS_Strip);  // Init of the Fastled library
      FastLED.addLeds<WS2812,41,RGB>(leds, 17*NUM_LEDS_Strip,NUM_LEDS_Strip);  // Init of the Fastled library
      FastLED.addLeds<WS2812,42,RGB>(leds, 18*NUM_LEDS_Strip,NUM_LEDS_Strip);  // Init of the Fastled library
      FastLED.addLeds<WS2812,43,RGB>(leds, 19*NUM_LEDS_Strip,NUM_LEDS_Strip);  // Init of the Fastled library
      FastLED.addLeds<WS2812,LED_PIN_Shoot,RGB>(leds_Shoot, NUM_LEDS_Strip_Shoot);  // Init of the Fastled library
    #endif
  #else
    //OpenFire 1 and Openfire2
    //Setup LEDs
    #if LEDTyp == 2812
      FastLED.addLeds<WS2812,LED_PIN,GRB>(leds, NUM_LEDS);  // Init of the Fastled library
      FastLED.addLeds<WS2812,LED_PIN_Shoot,GRB>(leds_Shoot, NUM_LEDS_Strip_Shoot);  // Init of the Fastled library
    #elif LEDTyp == 2811
      FastLED.addLeds<WS2811,LED_PIN,RGB>(leds, NUM_LEDS);  // Init of the Fastled library
      FastLED.addLeds<WS2811,LED_PIN_Shoot,RGB>(leds_Shoot, NUM_LEDS_Strip_Shoot);  // Init of the Fastled library
    #endif
  #endif


  #ifdef DEBUG
  Serial.println("----Init----");
  #endif
}

void loop() {

  Salut(0x010101, 80, 0);
  delay(500);
  Comet(0xFF0000, 0+(0*4), 80,0); //Comet front low, Red
  Comet(0x00FF00, 4+(2*4), 80,1); //Comet front left, Green
  delay(1000);
  Palm(0xFF3200, 64,1); // Fire
  TdV(1,0,1,60);//colortable(0..1),density(0)=no random, (1..9), sustain(0..1)=0=off, delay(ms)//red green
  Palm(0XFFFFFF, 64,2); // Rainbow
  Crackling(0xFF0000, 64,1); // Red
  Paeony(0XFF0000, 64,0);
  Palm(0XFFFFFF, 64,0); // Komet
  PalmGlitter(0xFF3200, 64,1); // Fire
  Sphere(0XFF3200, 0XFF3200, 128,0);
  PalmGlitter(0X00FF00, 64,0); // Rainbow
  Waterfall(0XFF3204, 128,1);
  TdV(1,6,1,60);//colortable(0..1),density(0)=no random, (1..9), sustain(0..1)=0=off, delay(ms)//red green
  PalmGlitter(0XFFFFFF, 64,0); // Komet
  TdV(0,6,1,80);//colortable(0..1),density(0)=no random, (1..9), sustain(0..1)=0=off, delay(ms) //TdV
  TdV(0,6,1,80);//colortable(0..1),density(0)=no random, (1..9), sustain(0..1)=0=off, delay(ms) //TdV
  Crackling(0XFFFFFF, 64,0); // Silver
  TdV(3,0,1,60);//colortable(0..1),density(0)=no random, (1..9), sustain(0..1)=0=off, delay(ms)//gold silver 
  Crackling(0XFF3210, 64,0); // 
  Paeony(0X00FF00, 64,0);
  Sphere(0X00FFFF, 0XFF3200, 128,0);
  Paeony(0X00FFFF, 64,0);
  Comet(0xFFFFFF, 0+(0*4), 80,2);
  TdV(3,6,1,60);//colortable(0..1),density(0)=no random, (1..9), sustain(0..1)=0=off, delay(ms)//gold silver
  Sphere(0X00FFFF, 0XFF00F0, 128,0);
  Waterfall(0XFF3200,128,1);
  TdV(2,6,1,60);//colortable(0..1),density(0)=no random, (1..9), sustain(0..1)=0=off, delay(ms)//gold
  Waterfall(0XFF3208, 128,1);
  TdV(2,0,1,60);//colortable(0..1),density(0)=no random, (1..9), sustain(0..1)=0=off, delay(ms)//gold
  Waterfall(0XFF3210, 128,1);
  TdV(0,0,1,80);//colortable(0..1),density(0)=no random, (1..9), sustain(0..1)=0=off, delay(ms)//red stripes
  Crackling(0XFFFFFF, 64,0); // Silver
  Salut(0x010101, 80, 0);
  delay(500);
  Salut(0x010101, 80, 0);
  delay(500);
  Salut(0x010101, 80, 0);
  delay (3000);

  #ifdef SOUND_ON
  myDFPlayer.play(15);  //applaus
  #endif
  

  delay (5000);
}
